import ExampleTheme from "./themes/ExampleTheme";
import {LexicalComposer} from "@lexical/react/LexicalComposer";
import {RichTextPlugin} from "@lexical/react/LexicalRichTextPlugin";
import {ContentEditable} from "@lexical/react/LexicalContentEditable";
import {HistoryPlugin} from "@lexical/react/LexicalHistoryPlugin";
import {AutoFocusPlugin} from "@lexical/react/LexicalAutoFocusPlugin";
import LexicalErrorBoundary from "@lexical/react/LexicalErrorBoundary";
import TreeViewPlugin from "./plugins/TreeViewPlugin";
import ToolbarPlugin from "./plugins/ToolbarPlugin";
import {Button} from "antd";
import {HeadingNode, QuoteNode} from "@lexical/rich-text";
import {TableCellNode, TableNode, TableRowNode} from "@lexical/table";
import {ListItemNode, ListNode} from "@lexical/list";
import {CodeHighlightNode, CodeNode} from "@lexical/code";
import {AutoLinkNode, LinkNode} from "@lexical/link";
import {LinkPlugin} from "@lexical/react/LexicalLinkPlugin";
import {ListPlugin} from "@lexical/react/LexicalListPlugin";
import {MarkdownShortcutPlugin} from "@lexical/react/LexicalMarkdownShortcutPlugin";
import {TRANSFORMERS} from "@lexical/markdown";
import "./index.less"
import ListMaxIndentLevelPlugin from "./plugins/ListMaxIndentLevelPlugin";
import CodeHighlightPlugin from "./plugins/CodeHighlightPlugin";
import AutoLinkPlugin from "./plugins/AutoLinkPlugin";
import {useEffect, useState} from 'react';
import {useLexicalComposerContext} from '@lexical/react/LexicalComposerContext';
import {importFile, overWriteFile} from "../../../utils/File"
import {CLEAR_HISTORY_COMMAND} from "lexical";
import ActionPlugin from "../../../components/ActionPlug"
import {func} from "prop-types";
import store from "../../../redux/store";
import {useSelector} from 'react-redux'
import {SAVE} from "../../../utils/HotkeyConst";
import md5 from "md5"

function Placeholder() {
    return <div className="editor-placeholder">Enter some rich text...</div>;
}

const editorConfig = {
    // The editor theme
    theme: ExampleTheme,
    // Handling of errors during update
    onError(error) {
        throw error;
    },
    // Any custom nodes go here
    nodes: [
        HeadingNode,
        ListNode,
        ListItemNode,
        QuoteNode,
        CodeNode,
        CodeHighlightNode,
        TableNode,
        TableCellNode,
        TableRowNode,
        AutoLinkNode,
        LinkNode
    ]
};

function OnChangePlugin({onChange}) {
    const [editor] = useLexicalComposerContext();
    console.log('editor', editor)
    useEffect(() => {
        return editor.registerUpdateListener(({editorState}) => {
            onChange(editorState);
        });
    }, [editor, onChange]);
}

function ImportFilePlugin(props) {
    const [editor] = useLexicalComposerContext();
    useEffect(() => {
        importFile(props.filePath).then(value => {
            const editorState = editor.parseEditorState(
                JSON.stringify(JSON.parse(value.toString()).editorState)
            );
            editor.setEditorState(editorState);
            editor.dispatchCommand(CLEAR_HISTORY_COMMAND, undefined);
        }).catch(error =>
            console.error(error)
        )
    }, [])
}

function SaveFilePlugin(props) {
    const [editor] = useLexicalComposerContext();
    useEffect(() => {
        store.subscribe(() => {
            let data = store.getState().pushHotkeys.data;
            const editorState = {"editorState":editor.getEditorState()};
            let resultSave = JSON.stringify(editorState);
            if (data === SAVE) {
                importFile(props.filePath).then(value => {
                    if (md5(resultSave)!==md5(JSON.stringify(JSON.parse(value.toString())))){
                        console.log("保存重写")
                        overWriteFile(props.filePath, resultSave)
                    }
                }).catch(error =>
                    console.error(error)
                )

            }
        })
    }, editor)
}

// 从字符串化 JSON 设置编辑器状态
// const editorState = editor.parseEditorState(editorStateJSONString);
// editor.setEditorState(editorState);

export default function Hlexical(props) {
    console.log("this.props.filePath:", props.filePath)
    const [editorState, setEditorState] = useState();

    function onChange(editorState) {
        // Call toJSON on the EditorState object, which produces a serialization safe string
        const editorStateJSON = editorState.toJSON();
        console.log('editorStateJSON', editorStateJSON)
        // However, we still have a JavaScript object, so we need to convert it to an actual string with JSON.stringify
        setEditorState(JSON.stringify(editorStateJSON));
    }

    return (
        <LexicalComposer initialConfig={editorConfig}>
            <div className="editor-container">
                {/* 富文本插件 */}
                <ToolbarPlugin/>
                <div className="editor-inner">
                    <RichTextPlugin
                        contentEditable={<ContentEditable className="editor-input"/>}
                        placeholder={<Placeholder/>}
                        ErrorBoundary={LexicalErrorBoundary}
                    />
                    <HistoryPlugin/>
                    {/*黑窗口动态记录当前操作*/}
                    <TreeViewPlugin/>
                    <AutoFocusPlugin/>
                    {/*<CodeHighlightPlugin />*/}
                    {/*<ListPlugin />*/}
                    {/*<LinkPlugin />*/}
                    {/*<AutoLinkPlugin />*/}
                    {/*<ListMaxIndentLevelPlugin maxDepth={7} />*/}
                    <MarkdownShortcutPlugin transformers={TRANSFORMERS}/>
                    <OnChangePlugin onChange={onChange}/>
                    <ImportFilePlugin filePath={props.filePath}/>
                    <SaveFilePlugin filePath={props.filePath}/>
                    {/*<ActionPlugin/>*/}
                </div>
            </div>
        </LexicalComposer>
    );
}
